@@ -31,7 +31,7 @@ import time
 import multiprocessing
 import concurrent.futures as conc
 
-from mesonbuild.mesonmain import backendlist
+from mesonbuild.coredata import backendlist
 
 class TestResult:
     def __init__(self, msg, stdo, stde, conftime=0, buildtime=0, testtime=0):
@@ -100,6 +100,11 @@ def setup_commands(backend):
         compile_commands = ['msbuild']
         test_commands = ['msbuild', 'RUN_TESTS.vcxproj']
         install_commands = []
+    elif backend == 'vs2015':
+        backend_flags = ['--backend=vs2015']
+        compile_commands = ['msbuild']
+        test_commands = ['msbuild', 'RUN_TESTS.vcxproj']
+        install_commands = []
     elif backend == 'xcode' or (backend is None and mesonlib.is_osx()):
         backend_flags = ['--backend=xcode']
         compile_commands = ['xcodebuild']
@@ -157,7 +162,7 @@ def validate_install(srcdir, installdir):
     return ''
 
 def log_text_file(logfile, testdir, stdo, stde):
-    global stop
+    global stop, executor, futures
     logfile.write('%s\nstdout\n\n---\n' % testdir)
     logfile.write(stdo)
     logfile.write('\n\n---\n\nstderr\n\n---\n')
@@ -167,6 +172,10 @@ def log_text_file(logfile, testdir, stdo, stde):
         print(stdo)
         print(stde, file=sys.stderr)
     if stop:
+        print("Aborting..")
+        for f in futures:
+            f[2].cancel()
+        executor.shutdown()
         raise StopException()
 
 def run_configure_inprocess(commandlist):
@@ -300,7 +309,7 @@ def detect_tests_to_run():
     return all_tests
 
 def run_tests(extra_args):
-    global passing_tests, failing_tests, stop
+    global passing_tests, failing_tests, stop, executor, futures
     all_tests = detect_tests_to_run()
     logfile = open('meson-test-run.txt', 'w', encoding="utf_8")
     junit_root = ET.Element('testsuites')
@@ -422,5 +431,8 @@ if __name__ == '__main__':
     print('\nTotal passed tests:', passing_tests)
     print('Total failed tests:', failing_tests)
     print('Total skipped tests:', skipped_tests)
+    if failing_tests > 0 and 'TRAVIS' in os.environ:
+        # Cat because it can have stuff of unknown encodings mixed.
+        subprocess.call(['cat', 'meson-test-run.txt'])
     sys.exit(failing_tests)
 
